searchData={"items":[{"type":"module","title":"Talos","doc":"Documentation for Talos.\n\nTalos is params type validation library, can be used with any Elixir application\n\nSample usage:\n\n```elixir\ndefmodule MyAppWeb.UserController do\n  # just import talos with functions helpers\n  import Talos\n\n  @interests_type enum(members: [\"sports\", \"games\", \"food\"]) # <- subtype\n  # here we define expected struct\n  @user_type map(fields: [\n    field(key: \"email\", type: string(min_length: 5, max_length: 255, regexp: ~r/.*@.*/)),\n    field(key: \"age\", type: integer(gteq: 18, allow_nil: true)),\n    field(key: \"interests\", type: list(type: @interests_type), optional: true)\n  ])\n\n  def create(conn, params) do\n    case Talos.valid?(@user_type, params) do\n      true ->\n        user = MyApp.create_user(params)\n        conn\n        |> put_flash(:info, \"User created successfully.\")\n        |> redirect(to: Routes.user_path(conn, :show, user))\n      false ->\n        conn\n        |> put_flash(:info, \"Wrong params passed.\")\n        |> render(\"new.html\")\n    end\n  end\nend\n```","ref":"Talos.html"},{"type":"function","title":"Talos.any/1","doc":"","ref":"Talos.html#any/1"},{"type":"function","title":"Talos.boolean/1","doc":"","ref":"Talos.html#boolean/1"},{"type":"function","title":"Talos.const/1","doc":"","ref":"Talos.html#const/1"},{"type":"function","title":"Talos.enum/1","doc":"","ref":"Talos.html#enum/1"},{"type":"function","title":"Talos.errors/2","doc":"","ref":"Talos.html#errors/2"},{"type":"function","title":"Talos.field/1","doc":"","ref":"Talos.html#field/1"},{"type":"function","title":"Talos.fixed/1","doc":"","ref":"Talos.html#fixed/1"},{"type":"function","title":"Talos.float/1","doc":"","ref":"Talos.html#float/1"},{"type":"function","title":"Talos.integer/1","doc":"","ref":"Talos.html#integer/1"},{"type":"function","title":"Talos.list/1","doc":"","ref":"Talos.html#list/1"},{"type":"function","title":"Talos.map/1","doc":"","ref":"Talos.html#map/1"},{"type":"function","title":"Talos.number/1","doc":"","ref":"Talos.html#number/1"},{"type":"function","title":"Talos.permit/2","doc":"Reduces `data` to what is defined in it's type.","ref":"Talos.html#permit/2"},{"type":"function","title":"Talos.string/1","doc":"","ref":"Talos.html#string/1"},{"type":"function","title":"Talos.valid?/2","doc":"","ref":"Talos.html#valid?/2"},{"type":"module","title":"Talos.Helpers.SchemaToMap","doc":"Schema to map, might be usefull for building documentation page","ref":"Talos.Helpers.SchemaToMap.html"},{"type":"function","title":"Talos.Helpers.SchemaToMap.convert/1","doc":"","ref":"Talos.Helpers.SchemaToMap.html#convert/1"},{"type":"behaviour","title":"Talos.Types","doc":"Talos Type behaviour\n\nYou can implement your own data type and use it with another Talos Types\n\n```elixir\n\n  iex> defmodule EmailType do\n  iex>   @behaviour Talos.Types\n  iex>   defstruct [] # Type must be struct, even empty\n  iex>   alias Talos.Types.StringType\n  iex>   @email_type %StringType{min_length: 5, regexp: ~r/.*@.*/, max_length: 255}\n  iex>   def valid?(_email_type, email) do\n  iex>     Talos.valid?(@email_type, email)\n  iex>   end\n  iex>   def errors(email_type, email) do\n  iex>     case valid?(email_type, email) do\n  iex>       true -> []\n  iex>       false -> [\"not valid email\"]\n  iex>     end\n  iex>   end\n  iex> end\n  iex> Talos.valid?(EmailType, \"example@foo.ru\")\n  true\n  iex> Talos.valid?(EmailType, \"example\")\n  false\n  iex> Talos.errors(EmailType, \"example@foo.ru\")\n  []\n  iex> Talos.errors(EmailType, \"example\")\n  [\"not valid email\"]\n\n```","ref":"Talos.Types.html"},{"type":"callback","title":"Talos.Types.errors/2","doc":"","ref":"Talos.Types.html#c:errors/2"},{"type":"callback","title":"Talos.Types.valid?/2","doc":"","ref":"Talos.Types.html#c:valid?/2"},{"type":"module","title":"Talos.Types.AnyType","doc":"Any type is used to show that data doesn't have a constant type\n\nFor example:\n```elixir\n  iex> import Talos, only: [any: 1]\n  iex> could_be_any_type = any()\n  iex> Talos.valid?(could_be_any_type, 2 == 2)\n  true\n  iex> Talos.valid?(could_be_any_type, nil)\n  true\n  iex> Talos.valid?(could_be_any_type, \"123\")\n  true\n```","ref":"Talos.Types.AnyType.html"},{"type":"function","title":"Talos.Types.AnyType.errors/2","doc":"","ref":"Talos.Types.AnyType.html#errors/2"},{"type":"function","title":"Talos.Types.AnyType.valid?/2","doc":"","ref":"Talos.Types.AnyType.html#valid?/2"},{"type":"type","title":"Talos.Types.AnyType.t/0","doc":"","ref":"Talos.Types.AnyType.html#t:t/0"},{"type":"module","title":"Talos.Types.BooleanType","doc":"Boolean type is used to check value is boolean\n\nFor example:\n```elixir\n  iex> import Talos, only: [boolean: 1]\n  iex> shoud_be_boolean = boolean(allow_nil: false)\n  iex> Talos.valid?(shoud_be_boolean, 2 == 2)\n  true\n  iex> Talos.valid?(shoud_be_boolean, \"true\")\n  false\n\n```\n\nAdditional parameters:\n\n`allow_nil` - allows value to be nil","ref":"Talos.Types.BooleanType.html"},{"type":"function","title":"Talos.Types.BooleanType.errors/2","doc":"","ref":"Talos.Types.BooleanType.html#errors/2"},{"type":"function","title":"Talos.Types.BooleanType.valid?/2","doc":"","ref":"Talos.Types.BooleanType.html#valid?/2"},{"type":"type","title":"Talos.Types.BooleanType.t/0","doc":"","ref":"Talos.Types.BooleanType.html#t:t/0"},{"type":"module","title":"Talos.Types.ConstType","doc":"Constant value validation","ref":"Talos.Types.ConstType.html"},{"type":"function","title":"Talos.Types.ConstType.errors/2","doc":"","ref":"Talos.Types.ConstType.html#errors/2"},{"type":"function","title":"Talos.Types.ConstType.valid?/2","doc":"","ref":"Talos.Types.ConstType.html#valid?/2"},{"type":"type","title":"Talos.Types.ConstType.t/0","doc":"","ref":"Talos.Types.ConstType.html#t:t/0"},{"type":"module","title":"Talos.Types.EnumType","doc":"Enum type is used to check value to be one of enumerable\n\nFor example:\n```elixir\n  iex> import Talos\n  iex> genders = enum(members: [\"male\", \"female\"])\n  iex> Talos.valid?(genders, \"male\")\n  true\n  iex> Talos.valid?(genders, \"female\")\n  true\n  iex> Talos.valid?(genders, \"heli\")\n  false\n\n```\n\n\nAlso there can be another Talos types:\n```elixir\n\n  iex> digit_string = %Talos.Types.StringType{regexp: ~r/^\\d+$/}\n  iex> numbers = %Talos.Types.EnumType{members: [digit_string, %Talos.Types.IntegerType{}]}\n  iex> Talos.valid?(numbers, \"1\")\n  true\n  iex> Talos.valid?(numbers, 1)\n  true\n  iex> Talos.valid?(numbers, \"One\")\n  false\n\n```\n\n\nAdditional parameters:\n\n`allow_nil` - allows value to be nil\n\n`members` - list of possible values or TalosTypes","ref":"Talos.Types.EnumType.html"},{"type":"function","title":"Talos.Types.EnumType.errors/2","doc":"","ref":"Talos.Types.EnumType.html#errors/2"},{"type":"function","title":"Talos.Types.EnumType.permit/2","doc":"","ref":"Talos.Types.EnumType.html#permit/2"},{"type":"function","title":"Talos.Types.EnumType.valid?/2","doc":"","ref":"Talos.Types.EnumType.html#valid?/2"},{"type":"type","title":"Talos.Types.EnumType.t/0","doc":"","ref":"Talos.Types.EnumType.html#t:t/0"},{"type":"module","title":"Talos.Types.FixedType","doc":"Fixed type is used to check value to be equal expected\n\nFor example:\n```elixir\n  iex> import Talos, only: [fixed: 1]\n  iex> the_4th_planet = fixed(value: \"Mars\")\n  iex> Talos.valid?(the_4th_planet, \"Earth\")\n  false\n  iex> Talos.valid?(the_4th_planet, \"Mars\")\n  true\n  \n```\n\n\nAdditional parameters:\n`allow_nil` - allows value to be nil","ref":"Talos.Types.FixedType.html"},{"type":"function","title":"Talos.Types.FixedType.errors/2","doc":"","ref":"Talos.Types.FixedType.html#errors/2"},{"type":"function","title":"Talos.Types.FixedType.valid?/2","doc":"","ref":"Talos.Types.FixedType.html#valid?/2"},{"type":"type","title":"Talos.Types.FixedType.t/0","doc":"","ref":"Talos.Types.FixedType.html#t:t/0"},{"type":"module","title":"Talos.Types.FloatType","doc":"Type for check value is float\n\nFor example:\n```elixir\n  iex> import Talos, only: [float: 1]\n  iex> percents = float(gteq: 0, lteq: 100)\n  iex> Talos.valid?(percents, 42.0)\n  true\n  iex>Talos.valid?(percents, 136.0)\n  false\n\n```\n\nAdditional parameters:\n\n`allow_nil` - allows value to be nil\n\n`allow_blank` - allows value to be blank (0.0)\n\n`gteq` - greater than or equal, same as `>=`\n\n`lteq` - lower than or equal, same as `<=`\n\n`gt` - lower than, same as `>`\n\n`lt` - lower than, same as `<`","ref":"Talos.Types.FloatType.html"},{"type":"function","title":"Talos.Types.FloatType.errors/2","doc":"","ref":"Talos.Types.FloatType.html#errors/2"},{"type":"function","title":"Talos.Types.FloatType.valid?/2","doc":"","ref":"Talos.Types.FloatType.html#valid?/2"},{"type":"type","title":"Talos.Types.FloatType.t/0","doc":"","ref":"Talos.Types.FloatType.html#t:t/0"},{"type":"module","title":"Talos.Types.IntegerType","doc":"Type for check value is integer\n\nFor example:\n```elixir\n  iex> import Talos, only: [integer: 1]\n  iex> percents = integer(gteq: 0, lteq: 100)\n  iex> Talos.valid?(percents, 42)\n  true\n  iex> Talos.valid?(percents, -15)\n  false\n  iex> Talos.valid?(percents, 30.0)\n  false\n\n```\n\nAdditional parameters:\n\n`allow_nil` - allows value to be nil\n\n`allow_blank` - allows value to be blank (0)\n\n`gteq` - greater than or equal, same as `>=`\n\n`lteq` - lower than or equal, same as `<=`\n\n`gt` - lower than, same as `>`\n\n`lt` - lower than, same as `<`","ref":"Talos.Types.IntegerType.html"},{"type":"function","title":"Talos.Types.IntegerType.errors/2","doc":"","ref":"Talos.Types.IntegerType.html#errors/2"},{"type":"function","title":"Talos.Types.IntegerType.valid?/2","doc":"","ref":"Talos.Types.IntegerType.html#valid?/2"},{"type":"type","title":"Talos.Types.IntegerType.t/0","doc":"","ref":"Talos.Types.IntegerType.html#t:t/0"},{"type":"module","title":"Talos.Types.ListType","doc":"ListType is used to check passed value is a list\n\n```elixir\n  iex> import Talos\n  iex> Talos.valid?(list(allow_blank: true), [])\n  true\n  iex> Talos.valid?(list(allow_nil: true), nil)\n  true\n  iex> Talos.valid?(list(), nil)\n  false\n  iex> Talos.valid?(list(), [\"one\", two, 3, %{}])\n  true\n  iex> Talos.valid?(list(type: integer()), [\"one\", two, 3, %{}])\n  false\n  iex> Talos.valid?(list(type: integer()), [1,2,3])\n  true\n  iex> Talos.valid?(list(type: integer(allow_nil: true)), [nil,2,3])\n  true\n\n```\n\nAdditional parameters:\n\n`allow_blank` - allows array to be empty\n\n`allow_nil` - allows value to be nil\n\n`type` - defines type of array elements","ref":"Talos.Types.ListType.html"},{"type":"function","title":"Talos.Types.ListType.errors/2","doc":"","ref":"Talos.Types.ListType.html#errors/2"},{"type":"function","title":"Talos.Types.ListType.permit/2","doc":"","ref":"Talos.Types.ListType.html#permit/2"},{"type":"function","title":"Talos.Types.ListType.valid?/2","doc":"","ref":"Talos.Types.ListType.html#valid?/2"},{"type":"type","title":"Talos.Types.ListType.t/0","doc":"","ref":"Talos.Types.ListType.html#t:t/0"},{"type":"module","title":"Talos.Types.MapType","doc":"MapType for validation maps\n\n`custom options`:\n\n    * `required_any_one`: true/false, if true - validation check if map has any valid field\n    * `required_groups`: list(\"key1\", \"key2\", ...), list of keys from required group,\n     required group can be simple field or field with dependendencies,\n     you need to specify all field from the group to check dependencies\n\nFields are tuples `{key, type, options \\ []}`:\n\n  `key` - string or atom, key of map\n\n  `type` - Talos defined Type\n\n  `options`:\n\n    * `optional`: true/false, if false - there will be error on key missing\n    * `depends_on`: list(\"key1\", \"key2\", ...), list of keys on which field depends,\n      field pass validation only if this field and all dependent fields pass validation\n\nFor example:\n```elixir\n\n  iex> import Talos\n  iex> Talos.valid?(map(), %{foo: :bar})\n  true\n  iex> user_params = map(fields: [\n  ...>  field(key: \"email\", type: string(min_length: 5, max_length: 255, regexp: ~r/.*@.*/)),\n  ...>  field(key: \"age\", type: integer(gteq: 18, allow_nil: true)),\n  ...>  field(key: \"interests\", type: list(type: string()), optional: true)\n  ...> ])\n  iex> Talos.valid?(user_params, %{})\n  false\n  iex> Talos.valid?(user_params, %{\"email\" => \"bob@gmail.com\", \"age\" => 30})\n  true\n  iex> Talos.valid?(user_params, %{\"email\" => \"bob@gmail.com\", \"age\" => 30, interests: [\"elixir\"]})\n  true\n  iex> deps = map(fields: [\n  ...>  field(\n  ...>   key: \"lastname\",\n  ...>   type: string(),\n  ...>   depends_on: [\"birthdate\", \"firstname\"]\n  ...>  ),\n  ...>  field(\n  ...>   key: \"firstname\",\n  ...>   type: string(),\n  ...>   depends_on: [\"birthdate\", \"lastname\"]\n  ...>  ),\n  ...>  field(\n  ...>   key: \"birthdate\",\n  ...>   type: string(),\n  ...>   depends_on: [\"lastname\", \"firstname\"]\n  ...>  )\n  ...>  ])\n  iex> Talos.valid?(deps, %{})\n  false\n  iex> Talos.valid?(deps, %{\"firstname\" => \"John\", \"lastname\" => \"Galt\"})\n  false\n  iex> Talos.valid?(deps, %{\"firstname\" => \"John\", \"lastname\" => \"Galt\", \"birthdate\" => \"1957-01-01\"})\n  true","ref":"Talos.Types.MapType.html"},{"type":"function","title":"Talos.Types.MapType.errors/2","doc":"","ref":"Talos.Types.MapType.html#errors/2"},{"type":"function","title":"Talos.Types.MapType.permit/2","doc":"","ref":"Talos.Types.MapType.html#permit/2"},{"type":"function","title":"Talos.Types.MapType.valid?/2","doc":"","ref":"Talos.Types.MapType.html#valid?/2"},{"type":"type","title":"Talos.Types.MapType.t/0","doc":"","ref":"Talos.Types.MapType.html#t:t/0"},{"type":"module","title":"Talos.Types.NumberType","doc":"Type for check value is number\n\nFor example:\n```elixir\n\n  iex> import Talos, only: [number: 1]\n  iex> percents = number(gteq: 0, lteq: 100)\n  iex> Talos.valid?(percents, 42)\n  true\n  iex> Talos.valid?(percents, -15)\n  false\n  iex> Talos.valid?(percents, 30.0)\n  true\n\n```\nAdditional parameters:\n\n`allow_nil` - allows value to be nil\n\n`gteq` - greater than or equal, same as `>=`\n\n`lteq` - lower than or equal, same as `<=`\n\n`gt` - lower than, same as `>`\n\n`lt` - lower than, same as `<`","ref":"Talos.Types.NumberType.html"},{"type":"function","title":"Talos.Types.NumberType.errors/2","doc":"","ref":"Talos.Types.NumberType.html#errors/2"},{"type":"function","title":"Talos.Types.NumberType.valid?/2","doc":"","ref":"Talos.Types.NumberType.html#valid?/2"},{"type":"type","title":"Talos.Types.NumberType.t/0","doc":"","ref":"Talos.Types.NumberType.html#t:t/0"},{"type":"module","title":"Talos.Types.StringType","doc":"Type for check value is string\n\nFor example:\n```elixir\n  iex> import Talos, only: [string: 1]\n  iex> short_domain = string(length: 3)\n  iex> domains_list = [\"cats\", \"foo\", \"baz\", \"pron\"]\n  iex> Enum.filter(domains_list, fn str -> Talos.valid?(short_domain, str) end)\n  [\"foo\", \"baz\"]\n\n```\n\nAdditional parameters:\n\n`min_length`, same as `String.length(str) <= max_length`\n\n`max_length`, same as `String.length(str) >= max_length`\n\n`length`, same as `String.length(str) >= length`\n\n`regexp`, same as `String.match?(str, regexp)`","ref":"Talos.Types.StringType.html"},{"type":"function","title":"Talos.Types.StringType.errors/2","doc":"","ref":"Talos.Types.StringType.html#errors/2"},{"type":"function","title":"Talos.Types.StringType.valid?/2","doc":"","ref":"Talos.Types.StringType.html#valid?/2"},{"type":"type","title":"Talos.Types.StringType.t/0","doc":"","ref":"Talos.Types.StringType.html#t:t/0"},{"type":"extras","title":"Talos","doc":"# Talos\n\n[![hex.pm](https://img.shields.io/badge/docs-hexpm-blue.svg)](https://hexdocs.pm/talos)\n[![hex.pm](https://img.shields.io/hexpm/v/talos.svg)](https://hex.pm/packages/talos)\n[![hex.pm](https://img.shields.io/hexpm/dt/talos.svg)](https://hex.pm/packages/talos)\n[![hex.pm](https://img.shields.io/hexpm/l/talos.svg)](https://hex.pm/packages/talos)\n[![github.com](https://img.shields.io/github/last-commit/balance-platform/talos.svg)](https://github.com/balance-platform/talos/commits/master)\n![Elixir CI](https://github.com/balance-platform/talos/workflows/Elixir%20CI/badge.svg)\n\nTalos is simple parameters validation library\n\nDocumentation can be found at [ExDoc](https://hexdocs.pm/talos/)","ref":"readme.html"},{"type":"extras","title":"Why another one validation library? - Talos","doc":"I needed more checks than just whether the value belonged to a particular data type. \n\nThis library allows you to define your own checks and use typical checks with a simple setup.","ref":"readme.html#why-another-one-validation-library"},{"type":"extras","title":"Usage - Talos","doc":"```elixir\n  defmodule CheckUserSignUp do\n    import Talos\n\n    @schema map(fields: [\n      field(key: \"action\", type: const(value: \"sign-up\")),\n      field(key: \"email\", type: string(min_length: 5, max_length: 255, regexp: ~r/.*@.*/)),\n      field(key: \"age\", type: integer(gteq: 18, allow_nil: true))\n    ])\n\n    def validate(%{} = map_data) do\n      params = Talos.permit(map_data)\n      errors = Talos.errors(@schema, params)\n\n      case errors == %{} do\n        true -> {:ok, params}\n        false -> {:error, errors}\n      end\n    end\n  end\n```\n\nSomewhere in UserController\n```elixir\n\n  ...\n\n  def sign_up(conn, params)\n    case CheckUserSignUp.valid?(params) do\n       :ok -> \n          result = MyApp.register_user!(params)\n          render_json(%{\"ok\" => true | result})\n       {:error, errors} -> \n          render_errors(errors)\n    end\n  end\n  \n  ...\n```","ref":"readme.html#usage"},{"type":"extras","title":"Flow - Talos","doc":"![](/.github/images/main_steps.png)","ref":"readme.html#flow"},{"type":"extras","title":"Own Type definition - Talos","doc":"If you want define own Type, just create module with `Talos.Types` behavior\n\n```elixir\ndefmodule ZipCodeType do\n  @behaviour Talos.Types\n  defstruct [length: 6]\n\n  def valid?(%__MODULE__{length: len}, value) do\n    String.valid?(value) && String.match?(value, ~r/\\d{len}/)\n  end\n\n  def errors(__MODULE__, value) do\n    case valid?(__MODULE__,value) do\n      true -> []\n      false -> [\"#{value} is not zipcode\"]\n    end\n  end\nend\n\n# And use it\n\nTalos.valid?(%ZipCodeType{}, \"123456\") # => true\nTalos.valid?(%ZipCodeType{}, \"1234\") # => false\nTalos.valid?(%ZipCodeType{}, 123456) # => false\n```","ref":"readme.html#own-type-definition"},{"type":"extras","title":"Installation - Talos","doc":"```elixir\ndef deps do\n  [\n    {:talos, \"~> 1.12\"}\n  ]\nend\n```\n\n# Contribution\n\nFeel free to make a pull request. All contributions are appreciated!","ref":"readme.html#installation"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,52,46,48]}}